use axum::{
    extract::State,
    http::HeaderMap,
    routing::{get, post},
    Json, Router,
};

use crate::{auth::get_user_from_header, model::Member, AppState};

pub mod migrate;

pub fn routes() -> Router<AppState> {
    Router::new()
        .route("/members/migrate_request", post(migrate::migrate_request))
        .route("/members/migrate_confirm", post(migrate::migrate_confirm))
        .route("/member", get(get_current_members))
}

pub async fn get_current_members(
    State(state): State<AppState>,
    headers: HeaderMap,
) -> Result<Json<Vec<Member>>, crate::Error> {
    let user = get_user_from_header(&state.pool, &headers).await?;

    let members = user.members(&state.pool).await?;

    Ok(Json(members))
}
